"""Kiso error classes."""


class KisoError(Exception):
    """KisoError _summary_."""


class KisoValueError(KisoError):
    """KisoValueError _summary_."""


class KisoTimeoutError(KisoError):
    """KisoTimeoutError _summary_."""
